from enum import Enum
from typing import Optional
from pydantic import BaseModel, Field
from datetime import datetime


# ============================= ENUM =============================


class ManufacturerChoices(str, Enum):
    # TODO: DeviceTypeModel Manufacturer to use name of type ManufacturerChoices
    ARISTA = "Arista"
    JUNIPER = "Juniper"
    DELL = "Dell"


class DeviceStatusChoices(str, Enum):
    STATUS_OFFLINE = "offline"
    STATUS_ACTIVE = "active"
    STATUS_PLANNED = "planned"
    STATUS_STAGED = "staged"
    STATUS_FAILED = "failed"
    STATUS_INVENTORY = "inventory"
    STATUS_DECOMMISSIONING = "decommissioning"


class IPAddressStatusChoices(str, Enum):
    STATUS_ACTIVE = "active"
    STATUS_RESERVED = "reserved"
    STATUS_DEPRECATED = "deprecated"
    STATUS_DHCP = "dhcp"
    STATUS_SLAAC = "slaac"


class DeviceRole(str, Enum):
    ACCESS_SWITCH = "Access Switch"
    CORE_SWITCH = "Core Switch"
    DISTRIBUTION_SWITCH = "Distribution Switch"
    FIREWALL = "Firewall"
    KVM = "KVM"

    @classmethod
    def from_string(cls, device_type):
        """
        Converts a string to the corresponding DeviceRole enum, if it exists.

        Args:
            device_type (str): The device type as a string.

        Returns:
            DeviceRole: The corresponding enum value.

        Raises:
            ValueError: If the device type is not found.
        """
        type_mapping = {
            "access": cls.ACCESS_SWITCH,
            "core": cls.CORE_SWITCH,
            "dist": cls.DISTRIBUTION_SWITCH,
            "firewall": cls.FIREWALL,
            "kvm": cls.KVM,
        }
        device_type_cleaned = device_type.strip().lower()
        if device_type_cleaned not in type_mapping:
            raise ValueError(f"Unknown device type/role: '{device_type}'")
        return type_mapping[device_type_cleaned]


# ============================= Helper models =============================
class Manufacturer(BaseModel):
    name: str


class Role(BaseModel):
    name: DeviceRole


class VRF(BaseModel):
    name: str


class Tenant(BaseModel):
    name: str


class Site(BaseModel):
    name: str


class ModelManufacturerDeviceType(BaseModel):
    manufacturer: Manufacturer
    model: str


class DeviceName(BaseModel):
    name: str


# ============================= Main models =============================


class DeviceTypeModel(BaseModel):
    manufacturer: Manufacturer
    model: str
    slug: str
    part_number: str
    u_height: int = Field(default=1, description="Default height of 1U for the device")
    comments: Optional[str]

    @classmethod
    def create(cls, manufacturer: str, model: str, part_number: str):
        """
        Creates a DeviceTypeModel instance with the given parameters.

        Args:
            manufacturer (str): The name of the manufacturer.
            model (str): The device model.
            part_number (str): The part number.
            formatted_date (str): The formatted date string.

        Returns:
            DeviceTypeModel: The populated model.
        """
        formatted_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        return cls(
            manufacturer=Manufacturer(name=manufacturer),
            model=model,
            slug=model.lower(),
            part_number=part_number,
            comments=f"{model} created by the sync-script-test at {formatted_date}",
        )


class DeviceModel(BaseModel):
    name: str
    device_type: ModelManufacturerDeviceType
    role: Role
    tenant: Tenant
    serial: str
    site: Site
    status: DeviceStatusChoices
    comments: Optional[str]

    def model_dump(self):
        data = super().model_dump()

        # Convert all enum values to their string representation
        for field, value in data.items():
            if isinstance(value, Enum):
                data[field] = value.value
            elif isinstance(value, BaseModel):
                # If it's a nested model, we can recursively call model_dump on it
                data[field] = value.model_dump()
            elif isinstance(value, dict):
                # If the value is a dictionary, check if it contains an Enum
                for sub_field, sub_value in value.items():
                    if isinstance(sub_value, Enum):
                        value[sub_field] = sub_value.value
                    elif isinstance(sub_value, BaseModel):
                        value[sub_field] = sub_value.model_dump()
        return data

    @classmethod
    def create(
        cls,
        name: str,
        role: DeviceRole,
        site: str,
        type: str,
        tenant: str,
        manufacturer: str,
        status: DeviceStatusChoices,
        serial: str,
    ):
        """
        Create a Device instance with the given parameters.

        Args:
            device (object): The device object (assumed to have attributes like name, manufacturer, etc.).
            formatted_date (str): The formatted date string.

        Returns:
            Device: The populated model.
        """
        formatted_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        return cls(
            name=name,
            device_type=ModelManufacturerDeviceType(
                manufacturer=Manufacturer(name=manufacturer), model=type
            ),
            role=Role(name=role),
            tenant=Tenant(name=tenant),
            serial=serial,
            site=Site(name=site),
            status=status,
            comments=f"Device {name} created by the sync-script-test at {formatted_date}",
        )


class InterfaceModel(BaseModel):
    # Add optional id for dry-run and testing
    id: Optional[int] = -1
    name: str = "MGMT"
    type: str = "virtual"
    device: DeviceName


class IpAddressModel(BaseModel):
    address: str
    vrf: VRF
    tenant: Tenant
    status: IPAddressStatusChoices
    comments: Optional[str]
    assigned_object_type: Optional[str]
    assigned_object_id: Optional[int]

    @classmethod
    def create(
        cls,
        address: str,
        vrf: str,
        tenant: str,
        status: IPAddressStatusChoices,
        assigned_object_type: str,
        assigned_object_id: int,
    ):
        formatted_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        ip_address = cls(
            address=address,
            vrf=VRF(name=vrf),
            tenant=Tenant(name=tenant),
            status=status,
            comments=f"IP {address} created by the sync-script-test at {formatted_date}",
            assigned_object_type=assigned_object_type,
            assigned_object_id=assigned_object_id,
        )
        return ip_address
